/*
$Id: f2cRts0.h,v 1.1.1.1 1998/07/21 10:27:53 rene Exp $
*/
#ifndef F2CRTS0
#define F2CRTS0

#include <f2c/r_const.h>
#include <f2c/r_type.h>

struct FieldType;
struct Field;
struct Member;
struct Obj;
struct Locator;

#if defined(__cplusplus) && !defined(FGLCEXT)
#define __FGLISLIB__
#endif

typedef struct Obj* ObjectP;
typedef long        Date;

#include <f2c/d_dec.h>

typedef struct Locator*  Text;
typedef struct Locator*  Byte;

#if defined(__FGLISLIB__) 

#include <f2c/r_dtime.h>

#else

typedef struct Datetime{
  short dt_qual;
  dec_t dt_dec;
}Datetime;

typedef struct Interval{
  short in_qual;
  dec_t in_dec;
}Interval;

typedef struct FieldType{
  Type ftType; /* Field */
  int  ftSize; /* Field */
  int  ftDim;  /* Array, Table */
  const struct FieldType* ftElementType; /* Array */
  const struct Member* ftMemberList;     /* Table */
  /* short subOffs, subSize;*/		/* SubString */
}FieldType;

#endif

#ifdef __FGLISLIB__
#include "r_field.h"
#else
typedef struct Field{
  void*            data;
  const FieldType* type;
}Field;

typedef struct Member{
  int              offset;
  const FieldType* type;
  const char*      ident;
}Member;

#endif

#ifdef __FGLISLIB__
#else
typedef struct Locator{
  void *loc;
}Locator;
#endif

#define OFFSET(t,x)     (int)&(((t*)0)->x)
#ifdef _AIX
#define OFFSETX(t,x)    (int)&(((t*)0)->x)
#else
#define OFFSETX(t,x)    (int)(((t*)0)->x)
#endif

#define LOCAL_INIT_FIELD(t,n)     op_fieldConstruct(&n ,     &F_##t,& C_##n)
#define LOCAL_INIT_FIELD_X(t,n)   op_fieldConstruct(&n ,     &F_##t,  C_##n)
#define LOCAL_INIT_RECORD(t,n)    op_fieldConstruct(&n.Table,&F_##t,& C_##n)

#define LOCAL_FIELD(t,n)          T_##t C_##n;        Field n 
#define LOCAL_RECORD(t,n)         T_##t C_##n;        t     n 
#define STATIC_FIELD(t,n)  static T_##t C_##n; static Field n=X_##t(C_##n)
#define STATIC_RECORD(t,n) static T_##t C_##n; static t     n=X_##t(C_##n)
#define GLOBAL_FIELD(t,n)         T_##t C_##n;        Field n=X_##t(C_##n)
#define GLOBAL_RECORD(t,n)        T_##t C_##n;        t     n=X_##t(C_##n)
#define EXTERN_FIELD(t,n)  extern T_##t C_##n; extern Field n
#define EXTERN_RECORD(t,n) extern T_##t C_##n; extern t     n
#define MODULE_FIELD(t,n)  STATIC_FIELD(t,n)
#define MODULE_RECORD(t,n) STATIC_RECORD(t,n)

#define CHAR_CONST(n,v) static const char C_##n[]=v; static const FieldType T_##n={T_cChar,sizeof(C_##n)-1};static const Field n={ (void*) C_##n, &T_##n }
#define INT_CONST(n,v)  static long       C_##n=v;   static const Field n={ & C_##n, &F_long }
#define DBL_CONST(n,v)  static double     C_##n=v;   static const Field n={ & C_##n, &F_double }
#define DECL_CURSOR(n)  static ObjectP    C_##n;     static Field n={ & C_##n, &F_ObjectP }
#define KEY(x) 		x
#define CONTROL(x)	(x & 0x1f)

typedef int (*FglFunction)(int);

#define OP_FUNCTION_BEGIN int nRet=0
#define OP_FUNCTION_DEBUG(x) rts_debugFunction(x,#x,argC,__FILE__,__LINE__);
#define OP_CALL(f,n,r)    if(f(n)!=r)error1320()
#define OP_RET            goto functionEnd;
#define OP_RET_ALLOC(n)   op_retAlloc(nRet=n);goto functionEnd;
#define OP_FUNCTION_END   functionEnd:op_functionEnd();return nRet

typedef char* CharP;
typedef Field* FieldP;

#ifdef __FGLISLIB__
#define FGLLANG extern "C" 
#else
#define FGLLANG 
#endif

FGLLANG void error1320();
FGLLANG void op_AssF();
FGLLANG void op_AssR();
FGLLANG void op_arrSub();
FGLLANG void op_arrSub2();
FGLLANG void op_arrSub3();
FGLLANG void op_fieldConstruct(Field*f,const FieldType*t,void*buff);
FGLLANG void op_popFunctionArg(Field *);
FGLLANG void op_pushC(const Field*);
FGLLANG void op_pushConstChar(const char*);
FGLLANG void op_pushConstDbl(const double);
FGLLANG void op_pushConstInt(int val);
FGLLANG void op_pushF(int);
FGLLANG void op_pushFixChar(char*,int len);
FGLLANG void op_genList(int n);
FGLLANG void op_pushNull();
FGLLANG void op_pushV(Field*);
FGLLANG void op_recExpand();
FGLLANG void op_recMember(int);
FGLLANG void op_recThru(int,int);
FGLLANG void op_retAlloc(int);
FGLLANG void op_retLoad(int);
FGLLANG void rts_OpSubStr1();
FGLLANG void rts_OpSubStr2();
FGLLANG void rts_OpSubStr1Left();
FGLLANG void rts_OpSubStr2Left();
FGLLANG void rts_debugFunction(FglFunction f,const char*,int,const char*,int);
FGLLANG void op_functionEnd();
FGLLANG void rts_debugReturn();
FGLLANG void rts_Op1Clipp();
FGLLANG int  rts_sqlWarning();
FGLLANG void rts_errLog(const char* fn,int line);
FGLLANG void rts_errStop(const char* fn,int line);
FGLLANG int P_main(int);

#endif
